import errno
import os
import sys
import time

import fetcher
import scatter

plotdefs = {
    "filesync_ops": {
        "key": "ops",
        "name": "filesync operations / sec",
        "scale": 48,
        "metric": "sumSeries(statsd.production.filesync.server.*.*.request_finished.rate)",
    },
    "filesync_connections": {
        "key": "connections",
        "name": "filesync connections",
        "scale": 48,
        "ticks": 3,
        "metric": "sumSeries(statsd.production.filesync.server.*.connections_active.count)"
    },
    "filesync_transfers": {
        "key": "transfers",
        "name": "filesync transfers",
        "scale": 48,
        "ticks": 3,
        "metric": "sumSeries(statsd.production.filesync.server.*.request_instances.*ContentResponse.count)"
    },
    "filesync_cpu": {
        "key": "fs-cpu",
        "name": "filesync cpu",
        "scale": 48,
        "metric": "sumSeries(statsd.production.*.resources.filesync-worker-*.cpu.percent.value)"
    },
    "rpcdal_worker_cpu": {
        "key": "rpcw-cpu",
        "name": "rpcdal worker cpu",
        "scale": 48,
        "metric": "sumSeries(statsd.production.*.resources.rpcdb-worker-*.cpu.percent.value)"
    },
    "rpcdal_broker_cpu": {
        "key": "rpcb-cpu",
        "name": "rpcdal broker cpu",
        "scale": 48,
        "metric": "sumSeries(statsd.production.*.resources.rpcdb-broker-*.cpu.percent.value)"
    },
    "ssl_proxy_cpu": {
        "key": "ssl-cpu",
        "name": "ssl proxy cpu",
        "scale": 48,
        "metric": "sumSeries(statsd.production.*.resources.ssl-proxy-*.cpu.percent.value)"
    },
    "filesync_vsize": {
        "key": "fs-vsize",
        "name": "filesync vsize (M)",
        "scale": 1024 * 1024 * 48,
        "metric": "sumSeries(statsd.production.*.resources.filesync-worker-*.memory.rss.value)"
    },
    "rpcdal_worker_vsize": {
        "key": "rpcw-vsize",
        "name": "rpcdal worker vsize (M)",
        "scale": 1024 * 1024 * 48,
        "metric": "sumSeries(statsd.production.*.resources.rpcdb-worker-*.memory.rss.value)"
    },
    "rpcdal_broker_vsize": {
        "key": "rpcb-vsize",
        "name": "rpcdal broker vsize (M)",
        "scale": 1024 * 1024 * 48,
        "metric": "sumSeries(statsd.production.*.resources.rpcdb-broker-*.memory.rss.value)"
    },
    "ssl_proxy_vsize": {
        "key": "ssl-vsize",
        "name": "ssl proxy vsize (M)",
        "scale": 1024 * 1024 * 48,
        "metric": "sumSeries(statsd.production.*.resources.ssl-proxy-*.memory.rss.value)"
    },
}

copy = plotdefs.copy()
for key in copy:
    entry = copy[key].copy()
    entry['key'] += "_half"
    entry['scale'] /= 2
    plotdefs[key + '_half'] = entry

for key in copy:
    entry = copy[key].copy()
    entry['key'] += "_raw"
    entry['scale'] = 1
    plotdefs[key + '_raw'] = entry


def mkdir_p(path):
    try:
        os.makedirs(path)
    except OSError as exc:
        if exc.errno == errno.EEXIST:
            pass
        else: raise

def download_data(plot_list, from_ts, to_ts, data_dir, cookie_file):
    unique_plots = set(plot_list)
    mkdir_p(data_dir)
    time_parse_format = "%Y%m%d%H%M%S"
    time_graphite_format = "%H:%M_%Y%m%d"

    t = [time.strptime(x, time_parse_format) for x in (from_ts, to_ts)]
    graphite_t = [time.strftime(time_graphite_format, x) for x in t]

    for p in unique_plots:

        d = plotdefs[p]
        data_file = os.path.join(data_dir, d["key"])

        if os.path.exists(data_file):
            print "file already exists", data_file
        else:
            target = d["metric"]
            url = "https://graphite.ubunet.canonical.com/render/" \
                  "?rawData=true" \
                  "&from=%s&until=%s" \
                  "&target=%s" \
                  % (graphite_t[0], graphite_t[1], target)

            print "downloading", d, "from url", url

            data = fetcher.fetch(url, cookie_file).read()
            outfile = open(data_file, 'w')
            outfile.write(data)
            outfile.close()

if __name__ == '__main__':
    datafile = os.path.abspath(sys.argv[1])
    from_ts = sys.argv[2]
    to_ts = sys.argv[3]
    metrics = sys.argv[4:]
    data_dir = from_ts + "_" + to_ts
    cookie_file = "cookies.dat"

    download_data(metrics, from_ts, to_ts, data_dir, cookie_file)
    os.chdir(data_dir)

    files = ["%s:%s:%s:%s" % (
        plotdefs[f]["name"],
        plotdefs[f]["scale"],
        plotdefs[f].get("ticks", 0),
        plotdefs[f]["key"],
    ) for f in metrics]

    print "metrics", metrics, "files", files

    scatter.main(['scatter'] + files)

    scatter.savefig(datafile, orientation="landscape", format="png")


